@@ -28,25 +28,39 @@
 #include "gdbusintrospection.h"
 
 #include "glibintl.h"
+#include "gioalias.h"
 
 /**
  * SECTION:gdbusintrospection
- * @title: D-Bus Introspection Data
- * @short_description: Node and interface description data structures
+ * @title: Introspection XML
+ * @short_description: Parse and Generate Introspection XML
  * @include: gio/gio.h
  *
  * Various data structures and convenience routines to parse and
  * generate D-Bus introspection XML. Introspection information is
  * used when registering objects with g_dbus_connection_register_object().
  *
- * The format of D-Bus introspection XML is specified in the
+ * The format of D-BUs introspection XML is specified in the
  * <link linkend="http://dbus.freedesktop.org/doc/dbus-specification.html&num;introspection-format">D-Bus specification</link>.
  */
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-#define _MY_DEFINE_BOXED_TYPE(TypeName, type_name) \
-  G_DEFINE_BOXED_TYPE (TypeName, type_name, type_name##_ref, type_name##_unref)
+/* See also https://bugzilla.gnome.org/show_bug.cgi?id=449565 ... */
+#define _MY_DEFINE_BOXED_TYPE(TypeName, type_name)                     \
+  GType                                                                 \
+  type_name##_get_type (void)                                           \
+  {                                                                     \
+    static volatile gsize type_volatile = 0;                            \
+    if (g_once_init_enter (&type_volatile))                             \
+      {                                                                 \
+        GType type = g_boxed_type_register_static (g_intern_static_string (#TypeName),  \
+                                                   (GBoxedCopyFunc) type_name##_ref,    \
+                                                   (GBoxedFreeFunc) type_name##_unref); \
+        g_once_init_leave (&type_volatile, type);                       \
+      }                                                                 \
+    return (GType) type_volatile;                                       \
+  }
 
 _MY_DEFINE_BOXED_TYPE (GDBusNodeInfo,       g_dbus_node_info);
 _MY_DEFINE_BOXED_TYPE (GDBusInterfaceInfo,  g_dbus_interface_info);
@@ -420,10 +434,10 @@ g_dbus_node_info_unref (GDBusNodeInfo *info)
 /* ---------------------------------------------------------------------------------------------------- */
 
 static void
-g_dbus_annotation_info_set (ParseData            *data,
-                            GDBusAnnotationInfo  *info,
-                            const gchar          *key,
-                            const gchar          *value,
+g_dbus_annotation_info_set (ParseData                      *data,
+                            GDBusAnnotationInfo            *info,
+                            const gchar                    *key,
+                            const gchar                    *value,
                             GDBusAnnotationInfo **embedded_annotations)
 {
   info->ref_count = 1;
@@ -462,7 +476,9 @@ static void
 g_dbus_method_info_set (ParseData            *data,
                         GDBusMethodInfo      *info,
                         const gchar          *name,
+                        guint                 in_num_args,
                         GDBusArgInfo        **in_args,
+                        guint                 out_num_args,
                         GDBusArgInfo        **out_args,
                         GDBusAnnotationInfo **annotations)
 {
@@ -471,11 +487,17 @@ g_dbus_method_info_set (ParseData            *data,
   if (name != NULL)
     info->name = g_strdup (name);
 
-  if (in_args != NULL)
-    info->in_args = in_args;
+  if (in_num_args != 0)
+    {
+      //info->in_num_args = in_num_args;
+      info->in_args = in_args;
+    }
 
-  if (out_args != NULL)
-    info->out_args = out_args;
+  if (out_num_args != 0)
+    {
+      //info->out_num_args = out_num_args;
+      info->out_args = out_args;
+    }
 
   if (annotations != NULL)
     info->annotations = annotations;
@@ -485,6 +507,7 @@ static void
 g_dbus_signal_info_set (ParseData            *data,
                         GDBusSignalInfo      *info,
                         const gchar          *name,
+                        guint                 num_args,
                         GDBusArgInfo        **args,
                         GDBusAnnotationInfo **annotations)
 {
@@ -493,11 +516,16 @@ g_dbus_signal_info_set (ParseData            *data,
   if (name != NULL)
     info->name = g_strdup (name);
 
-  if (args != NULL)
-    info->args = args;
+  if (num_args != 0)
+    {
+      //info->num_args = num_args;
+      info->args = args;
+    }
 
   if (annotations != NULL)
-    info->annotations = annotations;
+    {
+      info->annotations = annotations;
+    }
 }
 
 static void
@@ -517,44 +545,66 @@ g_dbus_property_info_set (ParseData               *data,
     info->flags = flags;
 
   if (signature != NULL)
-    info->signature = g_strdup (signature);
+    {
+      info->signature = g_strdup (signature);
+    }
 
   if (annotations != NULL)
-    info->annotations = annotations;
+    {
+      info->annotations = annotations;
+    }
 }
 
 static void
 g_dbus_interface_info_set (ParseData            *data,
                            GDBusInterfaceInfo   *info,
                            const gchar          *name,
+                           guint                 num_methods,
                            GDBusMethodInfo     **methods,
+                           guint                 num_signals,
                            GDBusSignalInfo     **signals,
+                           guint                 num_properties,
                            GDBusPropertyInfo   **properties,
                            GDBusAnnotationInfo **annotations)
 {
   info->ref_count = 1;
 
   if (name != NULL)
-    info->name = g_strdup (name);
+    {
+      info->name = g_strdup (name);
+    }
 
-  if (methods != NULL)
-    info->methods = methods;
+  if (num_methods != 0)
+    {
+      //info->num_methods    = num_methods;
+      info->methods        = methods;
+    }
 
-  if (signals != NULL)
-    info->signals = signals;
+  if (num_signals != 0)
+    {
+      //info->num_signals    = num_signals;
+      info->signals        = signals;
+    }
 
-  if (properties != NULL)
-    info->properties = properties;
+  if (num_properties != 0)
+    {
+      //info->num_properties = num_properties;
+      info->properties     = properties;
+    }
 
   if (annotations != NULL)
-    info->annotations = annotations;
+    {
+      info->annotations = annotations;
+    }
 }
 
 static void
 g_dbus_node_info_set (ParseData            *data,
                       GDBusNodeInfo        *info,
                       const gchar          *path,
+                      guint                 num_interfaces,
                       GDBusInterfaceInfo  **interfaces,
+                      guint                 num_nodes,
                       GDBusNodeInfo       **nodes,
                       GDBusAnnotationInfo **annotations)
 {
@@ -566,22 +616,31 @@ g_dbus_node_info_set (ParseData            *data,
       /* TODO: relative / absolute path snafu */
     }
 
-  if (interfaces != NULL)
-    info->interfaces = interfaces;
+  if (num_interfaces != 0)
+    {
+      //info->num_interfaces = num_interfaces;
+      info->interfaces     = interfaces;
+    }
 
-  if (nodes != NULL)
-    info->nodes = nodes;
+  if (num_nodes != 0)
+    {
+      //info->num_nodes      = num_nodes;
+      info->nodes          = nodes;
+    }
 
   if (annotations != NULL)
-    info->annotations = annotations;
+    {
+      info->annotations = annotations;
+    }
+
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
 
 static void
-g_dbus_annotation_info_generate_xml (GDBusAnnotationInfo *info,
-                                     guint                indent,
-                                     GString             *string_builder)
+g_dbus_annotation_info_generate_xml (const GDBusAnnotationInfo  *info,
+                                     guint                       indent,
+                                     GString                    *string_builder)
 {
   guint n;
 
@@ -610,10 +669,10 @@ g_dbus_annotation_info_generate_xml (GDBusAnnotationInfo *info,
 }
 
 static void
-g_dbus_arg_info_generate_xml (GDBusArgInfo *info,
-                              guint         indent,
-                              const gchar  *extra_attributes,
-                              GString      *string_builder)
+g_dbus_arg_info_generate_xml (const GDBusArgInfo  *info,
+                              guint                indent,
+                              const gchar         *extra_attributes,
+                              GString             *string_builder)
 {
   guint n;
 
@@ -646,9 +705,9 @@ g_dbus_arg_info_generate_xml (GDBusArgInfo *info,
 }
 
 static void
-g_dbus_method_info_generate_xml (GDBusMethodInfo *info,
-                                 guint            indent,
-                                 GString         *string_builder)
+g_dbus_method_info_generate_xml (const GDBusMethodInfo  *info,
+                                 guint                   indent,
+                                 GString                *string_builder)
 {
   guint n;
 
@@ -686,9 +745,9 @@ g_dbus_method_info_generate_xml (GDBusMethodInfo *info,
 }
 
 static void
-g_dbus_signal_info_generate_xml (GDBusSignalInfo *info,
-                                 guint            indent,
-                                 GString         *string_builder)
+g_dbus_signal_info_generate_xml (const GDBusSignalInfo  *info,
+                                 guint                   indent,
+                                 GString                *string_builder)
 {
   guint n;
 
@@ -720,9 +779,9 @@ g_dbus_signal_info_generate_xml (GDBusSignalInfo *info,
 }
 
 static void
-g_dbus_property_info_generate_xml (GDBusPropertyInfo *info,
-                                   guint              indent,
-                                   GString           *string_builder)
+g_dbus_property_info_generate_xml (const GDBusPropertyInfo  *info,
+                                   guint                     indent,
+                                   GString                  *string_builder)
 {
   guint n;
   const gchar *access_string;
@@ -785,9 +844,9 @@ g_dbus_property_info_generate_xml (GDBusPropertyInfo *info,
  * Since: 2.26
  */
 void
-g_dbus_interface_info_generate_xml (GDBusInterfaceInfo *info,
-                                    guint               indent,
-                                    GString            *string_builder)
+g_dbus_interface_info_generate_xml (const GDBusInterfaceInfo  *info,
+                                    guint                      indent,
+                                    GString                   *string_builder)
 {
   guint n;
 
@@ -832,9 +891,9 @@ g_dbus_interface_info_generate_xml (GDBusInterfaceInfo *info,
  * Since: 2.26
  */
 void
-g_dbus_node_info_generate_xml (GDBusNodeInfo *info,
-                               guint          indent,
-                               GString       *string_builder)
+g_dbus_node_info_generate_xml (const GDBusNodeInfo  *info,
+                               guint                 indent,
+                               GString              *string_builder)
 {
   guint n;
 
@@ -1231,9 +1290,6 @@ parse_data_free (ParseData *data)
   parse_data_free_methods (data);
   parse_data_free_signals (data);
   parse_data_free_properties (data);
-  parse_data_free_interfaces (data);
-  parse_data_free_annotations (data);
-  parse_data_free_nodes (data);
 
   g_free (data);
 }
@@ -1289,8 +1345,8 @@ parser_start_element (GMarkupParseContext  *context,
       g_dbus_node_info_set (data,
                             parse_data_get_node (data, TRUE),
                             name,
-                            NULL,
-                            NULL,
+                            0, NULL,
+                            0, NULL,
                             NULL);
 
       /* push the currently retrieved interfaces and nodes on the stack and prepare new arrays */
@@ -1320,17 +1376,15 @@ parser_start_element (GMarkupParseContext  *context,
                                         attribute_values,
                                         error,
                                         G_MARKUP_COLLECT_STRING, "name", &name,
-                                        /* seen in the wild */
-                                        G_MARKUP_COLLECT_STRING | G_MARKUP_COLLECT_OPTIONAL, "version", NULL,
                                         G_MARKUP_COLLECT_INVALID))
         goto out;
 
       g_dbus_interface_info_set (data,
                                  parse_data_get_interface (data, TRUE),
                                  name,
-                                 NULL,
-                                 NULL,
-                                 NULL,
+                                 0, NULL,
+                                 0, NULL,
+                                 0, NULL,
                                  NULL);
 
     }
@@ -1351,16 +1405,14 @@ parser_start_element (GMarkupParseContext  *context,
                                         attribute_values,
                                         error,
                                         G_MARKUP_COLLECT_STRING, "name", &name,
-                                        /* seen in the wild */
-                                        G_MARKUP_COLLECT_STRING | G_MARKUP_COLLECT_OPTIONAL, "version", NULL,
                                         G_MARKUP_COLLECT_INVALID))
         goto out;
 
       g_dbus_method_info_set (data,
                               parse_data_get_method (data, TRUE),
                               name,
-                              NULL,
-                              NULL,
+                              0, NULL,
+                              0, NULL,
                               NULL);
 
       data->num_args = 0;
@@ -1389,7 +1441,7 @@ parser_start_element (GMarkupParseContext  *context,
       g_dbus_signal_info_set (data,
                               parse_data_get_signal (data, TRUE),
                               name,
-                              NULL,
+                              0, NULL,
                               NULL);
 
       data->num_args = 0;
@@ -1470,10 +1522,7 @@ parser_start_element (GMarkupParseContext  *context,
                                         G_MARKUP_COLLECT_INVALID))
         goto out;
 
-      if (strcmp (stack->next->data, "method") == 0)
-        is_in = TRUE;
-      else
-        is_in = FALSE;
+      is_in = FALSE;
       if (direction != NULL)
         {
           if (strcmp (direction, "in") == 0)
@@ -1622,7 +1671,9 @@ parser_end_element (GMarkupParseContext  *context,
       g_dbus_node_info_set (data,
                             parse_data_get_node (data, FALSE),
                             NULL,
+                            num_interfaces,
                             interfaces,
+                            num_nodes,
                             nodes,
                             steal_annotations (data));
 
@@ -1643,8 +1694,11 @@ parser_end_element (GMarkupParseContext  *context,
       g_dbus_interface_info_set (data,
                                  parse_data_get_interface (data, FALSE),
                                  NULL,
+                                 num_methods,
                                  methods,
+                                 num_signals,
                                  signals,
+                                 num_properties,
                                  properties,
                                  steal_annotations (data));
 
@@ -1662,7 +1716,9 @@ parser_end_element (GMarkupParseContext  *context,
       g_dbus_method_info_set (data,
                               parse_data_get_method (data, FALSE),
                               NULL,
+                              in_num_args,
                               in_args,
+                              out_num_args,
                               out_args,
                               steal_annotations (data));
     }
@@ -1676,6 +1732,7 @@ parser_end_element (GMarkupParseContext  *context,
       g_dbus_signal_info_set (data,
                               parse_data_get_signal (data, FALSE),
                               NULL,
+                              num_args,
                               args,
                               steal_annotations (data));
     }
@@ -1769,7 +1826,6 @@ g_dbus_node_info_new_for_xml (const gchar  *xml_data,
   GMarkupParser *parser;
   guint num_nodes;
   ParseData *data;
-  GDBusNodeInfo **ughret;
 
   ret = NULL;
   parser = NULL;
@@ -1792,9 +1848,7 @@ g_dbus_node_info_new_for_xml (const gchar  *xml_data,
                                      error))
     goto out;
 
-  if (!g_markup_parse_context_end_parse (context, error))
-    goto out;
-
+  GDBusNodeInfo **ughret;
   ughret = parse_data_steal_nodes (data, &num_nodes);
 
   if (num_nodes != 1)
@@ -1811,11 +1865,10 @@ g_dbus_node_info_new_for_xml (const gchar  *xml_data,
       for (n = 0; n < num_nodes; n++)
         {
           for (n = 0; n < num_nodes; n++)
-            {
-              g_dbus_node_info_unref (ughret[n]);
-              ughret[n] = NULL;
-            }
+            g_dbus_node_info_unref (&(ret[n]));
         }
+      g_free (ret);
+      ret = NULL;
     }
 
   ret = ughret[0];
@@ -1846,14 +1899,14 @@ g_dbus_node_info_new_for_xml (const gchar  *xml_data,
  * Since: 2.26
  */
 const gchar *
-g_dbus_annotation_info_lookup (GDBusAnnotationInfo **annotations,
-                               const gchar          *name)
+g_dbus_annotation_info_lookup (const GDBusAnnotationInfo **annotations,
+                               const gchar                *name)
 {
   guint n;
   const gchar *ret;
 
   ret = NULL;
-  for (n = 0; annotations != NULL && annotations[n] != NULL; n++)
+  for (n = 0; annotations != NULL && annotations[n]->key != NULL; n++)
     {
       if (g_strcmp0 (annotations[n]->key, name) == 0)
         {
@@ -1881,16 +1934,16 @@ g_dbus_annotation_info_lookup (GDBusAnnotationInfo **annotations,
  *
  * Since: 2.26
  */
-GDBusMethodInfo *
-g_dbus_interface_info_lookup_method (GDBusInterfaceInfo *info,
-                                     const gchar        *name)
+const GDBusMethodInfo *
+g_dbus_interface_info_lookup_method (const GDBusInterfaceInfo *info,
+                                     const gchar              *name)
 {
   guint n;
-  GDBusMethodInfo *result;
+  const GDBusMethodInfo *result;
 
   for (n = 0; info->methods != NULL && info->methods[n] != NULL; n++)
     {
-      GDBusMethodInfo *i = info->methods[n];
+      const GDBusMethodInfo *i = info->methods[n];
 
       if (g_strcmp0 (i->name, name) == 0)
         {
@@ -1920,16 +1973,16 @@ g_dbus_interface_info_lookup_method (GDBusInterfaceInfo *info,
  *
  * Since: 2.26
  */
-GDBusSignalInfo *
-g_dbus_interface_info_lookup_signal (GDBusInterfaceInfo *info,
-                                     const gchar        *name)
+const GDBusSignalInfo *
+g_dbus_interface_info_lookup_signal (const GDBusInterfaceInfo *info,
+                                     const gchar              *name)
 {
   guint n;
-  GDBusSignalInfo *result;
+  const GDBusSignalInfo *result;
 
   for (n = 0; info->signals != NULL && info->signals[n] != NULL; n++)
     {
-      GDBusSignalInfo *i = info->signals[n];
+      const GDBusSignalInfo *i = info->signals[n];
 
       if (g_strcmp0 (i->name, name) == 0)
         {
@@ -1959,16 +2012,16 @@ g_dbus_interface_info_lookup_signal (GDBusInterfaceInfo *info,
  *
  * Since: 2.26
  */
-GDBusPropertyInfo *
-g_dbus_interface_info_lookup_property (GDBusInterfaceInfo *info,
-                                       const gchar        *name)
+const GDBusPropertyInfo *
+g_dbus_interface_info_lookup_property (const GDBusInterfaceInfo *info,
+                                       const gchar              *name)
 {
   guint n;
-  GDBusPropertyInfo *result;
+  const GDBusPropertyInfo *result;
 
   for (n = 0; info->properties != NULL && info->properties[n] != NULL; n++)
     {
-      GDBusPropertyInfo *i = info->properties[n];
+      const GDBusPropertyInfo *i = info->properties[n];
 
       if (g_strcmp0 (i->name, name) == 0)
         {
@@ -1998,16 +2051,16 @@ g_dbus_interface_info_lookup_property (GDBusInterfaceInfo *info,
  *
  * Since: 2.26
  */
-GDBusInterfaceInfo *
-g_dbus_node_info_lookup_interface (GDBusNodeInfo *info,
-                                   const gchar   *name)
+const GDBusInterfaceInfo *
+g_dbus_node_info_lookup_interface (const GDBusNodeInfo *info,
+                                   const gchar         *name)
 {
   guint n;
-  GDBusInterfaceInfo *result;
+  const GDBusInterfaceInfo *result;
 
   for (n = 0; info->interfaces != NULL && info->interfaces[n] != NULL; n++)
     {
-      GDBusInterfaceInfo *i = info->interfaces[n];
+      const GDBusInterfaceInfo *i = info->interfaces[n];
 
       if (g_strcmp0 (i->name, name) == 0)
         {
@@ -2021,3 +2074,6 @@ g_dbus_node_info_lookup_interface (GDBusNodeInfo *info,
  out:
   return result;
 }
+
+#define __G_DBUS_INTROSPECTION_C__
+#include "gioaliasdef.c"
