@@ -30,6 +30,14 @@
 #include <windns.h>
 #include <mswsock.h>
 
+#ifdef HAVE_WSPIAPI_H
+/* <wspiapi.h> in the Windows SDK and in mingw-w64 has wrappers for
+ * inline workarounds for getaddrinfo, getnameinfo and freeaddrinfo if
+ * they aren't present at run-time (on Windows 2000).
+ */
+#include <wspiapi.h>
+#endif
+
 #else /* !G_OS_WIN32 */
 
 #include <sys/types.h>
@@ -56,6 +64,22 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 
+#ifndef _PATH_RESCONF
+#define _PATH_RESCONF "/etc/resolv.conf"
+#endif
+
+#ifndef CMSG_LEN
+/* CMSG_LEN and CMSG_SPACE are defined by RFC 2292, but missing on
+ * some older platforms.
+ */
+#define CMSG_LEN(len) ((size_t)CMSG_DATA((struct cmsghdr *)NULL) + (len))
+
+/* CMSG_SPACE must add at least as much padding as CMSG_NXTHDR()
+ * adds. We overestimate here.
+ */
+#define ALIGN_TO_SIZEOF(len, obj) (((len) + sizeof (obj) - 1) & ~(sizeof (obj) - 1))
+#define CMSG_SPACE(len) ALIGN_TO_SIZEOF (CMSG_LEN (len), struct cmsghdr)
+#endif
 #endif
 
 G_BEGIN_DECLS
@@ -69,7 +93,7 @@ GList *_g_resolver_addresses_from_addrinfo (const char       *hostname,
 
 void   _g_resolver_address_to_sockaddr     (GInetAddress            *address,
 					    struct sockaddr_storage *sa,
-					    gsize                   *sa_len);
+					    gsize                   *len);
 char  *_g_resolver_name_from_nameinfo      (GInetAddress     *address,
 					    const gchar      *name,
 					    gint              gni_retval,
