@@ -70,8 +70,9 @@
  * <tag class="starttag">key</tag> element.
  *
  * GSettings uses schemas in a compact binary form that is created
- * by the gschema-compile utility. The input is a schema description in
- * an XML format that can be described by the following DTD:
+ * by the <link linkend="glib-compile-schemas">glib-compile-schemas</link>
+ * utility. The input is a schema description in an XML format that can be
+ * described by the following DTD:
  * |[<![CDATA[
  * <!ELEMENT schemalist (schema*) >
  * <!ATTLIST schemalist gettext-domain #IMPLIED >
@@ -99,9 +100,9 @@
  * <!ELEMENT summary (#PCDATA) >
  * <!ELEMENT description (#PCDATA) >
  *
- * <!ELEMENT range (min,max)  >
- * <!ELEMENT min (#PCDATA) >
- * <!ELEMENT max (#PCDATA) >
+ * <!ELEMENT range EMPTY >
+ * <!ATTLIST range min CDATA #REQUIRED
+ *                 max CDATA #REQUIRED >
  *
  * <!ELEMENT choices (choice+) >
  * <!ELEMENT choice (alias?) >
@@ -750,7 +751,7 @@ g_settings_get_value (GSettings   *settings,
 
   path = g_strconcat (settings->priv->path, key, NULL);
   type = g_variant_get_type (sval);
-  value = g_settings_backend_read (settings->priv->backend, path, type);
+  value = g_settings_backend_read (settings->priv->backend, path, type, FALSE);
   g_free (path);
 
   if (options != NULL)
@@ -1448,6 +1449,8 @@ g_settings_writable_binding_free (gpointer data)
 
   g_signal_handler_disconnect (binding->settings, binding->handler_id);
   g_object_unref (binding->settings);
+  g_free (binding->key);
+  g_slice_free (GSettingsWritableBinding, binding);
 }
 
 static void
@@ -1528,6 +1531,7 @@ g_settings_bind_writable (GSettings   *settings,
   binding = g_slice_new (GSettingsWritableBinding);
   binding->settings = g_object_ref (settings);
   binding->object = object;
+  binding->key = g_strdup (key);
   binding->property = g_intern_string (property);
   binding->inverted = inverted;
 
@@ -1787,6 +1791,7 @@ g_settings_set_boolean (GSettings  *settings,
  * g_settings_get_strv:
  * @settings: a #GSettings object
  * @key: the key to get the value for
+ * @length: return location for the length of the result, or %NULL
  * @returns: a newly-allocated, %NULL-terminated array of strings
  *
  * Gets the value that is stored at @key in @settings.
@@ -1818,6 +1823,7 @@ g_settings_get_strv (GSettings   *settings,
  * @settings: a #GSettings object
  * @key: the name of the key to set
  * @value: the value to set it to
+ * @length: the length of the @value array, or -1
  * @returns: %TRUE if setting the key succeeded,
  *     %FALSE if the key was not writable
  *
